<?php

namespace Drupal\field_pager;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;

/**
 * Helper functions for field_pager.
 */
class PagerHelper {

  /**
   * Add default settings.
   */
  public static function mergeDefaultSettings($settings = []) {

    return [
      'index_name' => 'index',
      'pager_mode' => 0,
      'summary' => 0,
    ] + $settings;

  }

  /**
   * Add settings to the form.
   */
  public static function mergeSettingsForm(array $form, FormStateInterface $form_state, $instance, $elements = []) {
    $elements['summary'] = [
      '#type' => 'checkbox',
      '#title' => t('Show summary'),
      '#description' => t('Show text "Page X of N"'),
      '#default_value' => $instance->getSetting('summary'),
    ];
    $elements['index_name'] = [
      '#type' => 'textfield',
      '#title' => t('Index field name'),
      '#default_value' => $instance->getSetting('index_name'),
      '#required' => TRUE,
    ];
    $elements['pager_mode'] = [
      '#type' => 'select',
      '#options' => [0 => 'Next & Previus', 1 => "Items", 2 => "Full"],
      '#title' => t('Pager mode'),
      '#default_value' => $instance->getSetting('pager_mode'),
      '#required' => TRUE,
    ];

    return $elements;
  }

  /**
   * Add settings summary.
   */
  public static function mergeSettingsSummary($summary, $instance) {
    $pager_mode = $instance->getSetting('pager_mode');
    $options = [0 => 'Next & Previus', 1 => "Items", 2 => "Full"];
    $summary[] = "Pager : " . $options[$pager_mode];
    return $summary;
  }

  /**
   * Add View.
   */
  public static function mergeView($nb_items, $instance, $fields, $settings = []) {

    // Config.
    $index_name = $instance->getSetting('index_name');
    $pager_mode = $instance->getSetting('pager_mode');
    $summary = $instance->getSetting('summary');
    $index_current = (int) (isset($_GET[$index_name]) ? $_GET[$index_name] : 0);

    $elements = [];
    if ($summary) {
      $elements['summary'] = [
        '#markup' => new TranslatableMarkup("Page @index of @total", [
          '@index' => $index_current + 1,
          '@total' => $nb_items,
        ]),
        '#prefix' => '<div>',
        '#suffix' => '</div>',
      ];
    }
    $elements['fields'] = $fields;

    // Handle cache @todo
    $elements['#cache']['max-age'] = 0;

    // Set next / Previus index.
    $index_prev = $index_current ? $index_current - 1 : 0;
    $index_next = ($index_current + 1) < $nb_items ? ($index_current + 1) : 0;

    $items = [];

    $url = Url::fromRoute('<current>');

    if ($pager_mode == 0 || $pager_mode == 2) {

      $items[] = [
        'text' => new TranslatableMarkup("Previous"),
        'attributes' => [],
        'href' => $url->setRouteParameter($index_name, $index_prev)->toString(),
      ];

    }
    if ($pager_mode == 1 || $pager_mode == 2) {
      for ($i = 0; $i < $nb_items; $i++) {
        $attributes = [];
        if ($i == $index_current) {
          $attributes['class'][] = "active";
        }
        if ($i == 0) {
          $attributes['class'][] = "first";
        }
        if ($i == $nb_items - 1) {
          $attributes['class'][] = "last";
        }

        // Add $items.
        $items[] = [
          'text' => ($i + 1),
          'attributes' => $attributes,
          'href' => $url->setRouteParameter($index_name, $i)->toString(),
        ];

      }
    }

    if ($pager_mode == 0 || $pager_mode == 2) {
      $items[] = [
        'text' => new TranslatableMarkup("Next"),
        'attributes' => [],
        'href' => $url->setRouteParameter($index_name, $index_next)->toString(),
      ];
    }

    $elements['custom_pager'] = [
      '#theme' => 'field_pager',
      '#index' => $index_current,
      '#items' => $items,
      '#settings' => $settings,
    ];

    return $elements;
  }

}
